<?php

declare(strict_types=1);

namespace Drupal\log_middleware;

use Drupal\Core\Logger\LoggerChannel;
use Drupal\log_middleware\Middleware\LogMiddlewareInterface;
use Drupal\log_middleware\Middleware\LogMiddlewareResult;

/**
 * Extends the LoggerChannel with middleware support.
 */
class LoggerChannelWithMiddleware extends LoggerChannel {

  /**
   * Array of log middleware keyed by priority.
   *
   * @var array<int, LogMiddlewareInterface[]>
   */
  protected array $middleware = [];

  /**
   * {@inheritdoc}
   */
  public function log($level, string|\Stringable $message, array $context = []): void {
    // Apply middleware.
    // Middleware directly alters $level, $message and $context.
    // Additionally if a middleware returns a
    // LogMiddlewareResult::DiscardMessage, the message is not logged.
    foreach ($this->sortMiddleware() as $middleware) {
      $result = $middleware->alterLog($this->channel, $level, $message, $context);

      // Discard this log message.
      if ($result === LogMiddlewareResult::DiscardMessage) {
        return;
      }
    }

    // Log the message.
    parent::log($level, $message, $context);
  }

  /**
   * Sets the middleware for this channel.
   */
  public function setMiddleware(array $middleware) {
    $this->middleware = $middleware;
  }

  /**
   * Add a middleware for this channel.
   */
  public function addMiddleware(LogMiddlewareInterface $middleware, $priority = 0) {
    $this->middleware[$priority][] = $middleware;
  }

  /**
   * Sorts middleware according to priority.
   *
   * @return \Drupal\log_middleware\Middleware\LogMiddlewareInterface[]
   *   An array of sorted middleware by priority.
   */
  protected function sortMiddleware() {
    krsort($this->middleware);
    return array_merge(...$this->middleware);
  }

}
