<?php

declare(strict_types=1);

namespace Drupal\log_middleware;

use Drupal\Core\Logger\LoggerChannelFactory;
use Drupal\log_middleware\Middleware\LogMiddlewareInterface;

/**
 * Enhances the LoggerChannelFactory for LoggerChannelWithMiddleware.
 */
class LoggerChannelWithMiddlewareFactory extends LoggerChannelFactory {

  /**
   * Array of log middleware.
   *
   * @var array<int, array{middleware: LogMiddlewareInterface, channels: string[]}[]>
   */
  protected array $middleware = [];

  /**
   * {@inheritdoc}
   */
  public function get($channel) {
    // Check if the channel does not exist.
    if (!isset($this->channels[$channel])) {
      // Create a new instance of LoggerChannelWithMiddleware.
      // If the instance is set here, than a logger channel is
      // set in the parent class.
      $instance = new LoggerChannelWithMiddleware($channel);

      // Set the request_stack and current_user services on the channel.
      // It is up to the channel to determine if there is a current request.
      $instance->setRequestStack($this->requestStack);
      $instance->setCurrentUser($this->currentUser);

      // Pass the loggers to the channel.
      $instance->setLoggers($this->loggers);

      // Pass the middleware to the channel.
      $instance->setMiddleware($this->filterMiddleware($channel));

      $this->channels[$channel] = $instance;
    }

    // Call the parent function.
    return parent::get($channel);
  }

  /**
   * Filter middleware by given channel.
   *
   * Returns all middleware grouped by priority.
   */
  protected function filterMiddleware(string $channel) {
    $filtered = [];

    foreach ($this->middleware as $priority => $items) {
      foreach ($items as $item) {
        if (empty($item['channels']) || in_array($channel, $item['channels'])) {
          $filtered[$priority][] = $item['middleware'];
        }
      }
    }

    return $filtered;
  }

  /**
   * Add a middleware to all channels.
   */
  public function addMiddleware(LogMiddlewareInterface $middleware, $priority = 0, $channels = []) {
    // Store it so we can pass it to potential new logger instances.
    $this->middleware[$priority][] = ['middleware' => $middleware, 'channels' => $channels];

    // Add the middleware to already instantiated channels.
    foreach ($this->channels as $channel) {
      if ($channel instanceof LoggerChannelWithMiddleware && (empty($channels) || in_array($channel, $channels))) {
        $channel->addMiddleware($middleware, $priority);
      }
    }
  }

}
