<?php

declare(strict_types=1);

namespace Drupal\Tests\log_middleware\Kernel;

use Drupal\Core\Database\Connection;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\KernelTests\Core\Entity\EntityKernelTestBase;
use Drupal\log_middleware\LoggerChannelWithMiddleware;

/**
 * Test the logger.
 */
class LogMiddlewareTest extends EntityKernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'dblog',
    'log_middleware',
    'log_test_middleware',
  ];

  /**
   * The db connection.
   */
  private Connection $database;

  /**
   * Logger channel factory.
   */
  private LoggerChannelFactoryInterface $loggerFactory;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installSchema('dblog', 'watchdog');

    $this->database = $this->container->get('database');
    $this->loggerFactory = $this->container->get('logger.factory');
  }

  /**
   * Test that the logger channel has correct class.
   */
  public function testLoggerChannelClass() {
    $log = $this->loggerFactory->get('demo');

    $this->assertEquals(LoggerChannelWithMiddleware::class, get_class($log));
  }

  /**
   * Test that logging works.
   */
  public function testNormalLogging() {
    $msg = 'a log message';

    $log = $this->loggerFactory->get('demo');

    $log->debug($msg);
    $log->info($msg);
    $log->notice($msg);
    $log->warning($msg);
    $log->error($msg);
    $log->emergency($msg);
    $log->critical($msg);

    $this->assertEquals(7, $this->getLogCount());
  }

  /**
   * Test altered level.
   */
  public function testAlteredLevel() {
    $msg = 'a log message';

    // See LogTestMiddleware why this channel is set.
    $log = $this->loggerFactory->get('alter_level');

    $log->debug($msg);

    $this->assertEquals(1, $this->getLogCount());

    $latestLog = $this->getLatestLog();

    // 3 -> error
    $this->assertEquals(3, $latestLog->severity);
    $this->assertEquals($msg, $latestLog->message);
  }

  /**
   * Test altered message.
   */
  public function testAlteredMessage() {
    $msg = 'a log message';

    // See LogTestMiddleware why this channel is set.
    $log = $this->loggerFactory->get('alter_message');

    $log->debug($msg);

    $this->assertEquals(1, $this->getLogCount());

    $latestLog = $this->getLatestLog();

    // 7 -> debug
    $this->assertEquals(7, $latestLog->severity);
    $this->assertEquals('message from middleware', $latestLog->message);
  }

  /**
   * Test discarded log.
   */
  public function testDiscardedLog() {
    $msg = 'a log message';

    // See LogTestMiddleware why this channel is set.
    $log = $this->loggerFactory->get('discard');

    $log->debug($msg);

    $this->assertEquals(0, $this->getLogCount());
  }

  /**
   * Test channel specific middleware.
   */
  public function testChannelSpecificMiddleware() {
    $msg = 'a log message';

    // See LogChannelSpecificTestMiddleware why this channel is set.
    $log = $this->loggerFactory->get('specific');

    $log->debug($msg);

    $this->assertEquals(1, $this->getLogCount());

    $latestLog = $this->getLatestLog();

    // 4 -> warning
    $this->assertEquals(4, $latestLog->severity);
    $this->assertEquals('specific middleware altered', $latestLog->message);
  }

  /**
   * Get total number of logs in db.
   */
  protected function getLogCount() {
    $data = $this->database->query('SELECT count(*) FROM watchdog')->fetchAll();

    return (int) $data[0]->{'count(*)'};
  }

  /**
   * Get the latest log entry.
   */
  protected function getLatestLog() {
    $data = $this->database->query('SELECT * FROM watchdog ORDER BY wid DESC LIMIT 1')->fetchAll();

    return $data[0];
  }

}
